<?php

namespace Litecms\Series\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Series\Forms\Series as SeriesForm;
use Litecms\Series\Http\Requests\SeriesRequest;
use Litecms\Series\Interfaces\SeriesRepositoryInterface;
use Litecms\Series\Repositories\Eloquent\Filters\SeriesResourceFilter;
use Litecms\Series\Repositories\Eloquent\Presenters\SeriesListPresenter;

/**
 * Resource controller class for series.
 */
class SeriesResourceController extends BaseController
{

    /**
     * Initialize series resource controller.
     *
     *
     * @return null
     */
    public function __construct(SeriesRepositoryInterface $series)
    {
        parent::__construct();
        $this->form = SeriesForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.series.modules'), 'series', guard_url('series'));
        $this->repository = $series;
    }

    /**
     * Display a list of series.
     *
     * @return Response
     */
    public function index(SeriesRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(SeriesResourceFilter::class)
            ->setPresenter(SeriesListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('series::series.names'))
            ->view('series::series.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display series.
     *
     * @param Request $request
     * @param Model   $series
     *
     * @return Response
     */
    public function show(SeriesRequest $request, SeriesRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('series::series.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('series::series.show')
            ->output();
    }

    /**
     * Show the form for creating a new series.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(SeriesRequest $request, SeriesRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('series::series.name'))
            ->view('series::series.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new series.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SeriesRequest $request, SeriesRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('series::series.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('series/series/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/series/series'))
                ->redirect();
        }

    }

    /**
     * Show series for editing.
     *
     * @param Request $request
     * @param Model   $series
     *
     * @return Response
     */
    public function edit(SeriesRequest $request, SeriesRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('series::series.name'))
            ->view('series::series.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the series.
     *
     * @param Request $request
     * @param Model   $series
     *
     * @return Response
     */
    public function update(SeriesRequest $request, SeriesRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('series::series.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('series/series/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('series/series/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the series.
     *
     * @param Model   $series
     *
     * @return Response
     */
    public function destroy(SeriesRequest $request, SeriesRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('series::series.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('series/series/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('series/series/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
